package org.mercury_im.messenger.persistence.room.dao;

import androidx.room.Delete;
import androidx.room.Insert;
import androidx.room.OnConflictStrategy;
import androidx.room.Transaction;
import androidx.room.Update;

import java.util.List;

import io.reactivex.Completable;
import io.reactivex.Observable;
import io.reactivex.Single;

public interface BaseDao<T> {

    @Insert(onConflict = OnConflictStrategy.IGNORE)
    Single<Long> insert(T entity);

    @Insert(onConflict = OnConflictStrategy.IGNORE)
    Single<Long[]> insert(T[] entities);

    @Insert(onConflict = OnConflictStrategy.IGNORE)
    Single<List<Long>> insert(List<T> entities);

    @Update
    Completable update(T entity);

    @Update
    Completable update(T[] entities);

    @Delete
    Completable delete(T entity);

    @Delete
    Completable delete(T[] entities);

    @Delete
    Completable delete(List<T> entities);
}
