package org.mercury_im.messenger.persistence.room.repository;

import android.util.Log;

import androidx.annotation.NonNull;

import org.mercury_im.messenger.persistence.repository.AccountRepository;
import org.mercury_im.messenger.persistence.room.dao.AccountDao;
import org.mercury_im.messenger.persistence.room.model.RoomAccountModel;

import java.util.List;

import javax.inject.Inject;

import io.reactivex.Completable;
import io.reactivex.Maybe;
import io.reactivex.Observable;
import io.reactivex.Single;

import static org.mercury_im.messenger.persistence.room.AppDatabase.TAG;

public class IAccountRepository implements AccountRepository<RoomAccountModel> {

    private final AccountDao accountDao;

    @Inject
    public IAccountRepository(AccountDao accountDao) {
        this.accountDao = accountDao;
    }

    @Override
    public RoomAccountModel newAccountModel() {
        return new RoomAccountModel();
    }

    @Override
    public Observable<RoomAccountModel> getAccount(long accountId) {
        return accountDao.getAccountById(accountId);
    }

    @Override
    public Maybe<RoomAccountModel> maybeGetAccount(long accountId) {
        return accountDao.maybeGetAccountById(accountId);
    }

    @Override
    public Observable<List<RoomAccountModel>> getAllAccounts() {
        return accountDao.getAllAccounts();
    }


    @Override
    public Single<Long> insertAccount(@NonNull RoomAccountModel accountModel) {
        return accountDao.insert(accountModel)
                .map(accountId -> {
                    accountModel.setId(accountId);
                    return accountId;
                })
                .doOnSubscribe(ignore -> Log.v(TAG, "Insert " + accountModel))
                .doAfterSuccess(accountId -> Log.v(TAG, "AccountModel has new ID " + accountId));
    }

    @Override
    public Completable updateAccount(RoomAccountModel accountModel) {
        return accountDao.update(accountModel);
    }

    @Override
    public Completable deleteAccount(long accountId) {
        return accountDao.deleteAccount(accountId);
    }

    @Override
    public Completable deleteAccount(RoomAccountModel item) {
        return accountDao.delete(item);
    }
}
