package org.mercury_im.messenger.persistence.repository;

import org.jxmpp.jid.EntityBareJid;
import org.mercury_im.messenger.persistence.model.AccountModel;
import org.mercury_im.messenger.persistence.model.ChatModel;
import org.mercury_im.messenger.persistence.model.ContactModel;
import org.mercury_im.messenger.persistence.model.EntityModel;
import org.mercury_im.messenger.persistence.pojo.Chat;

import java.util.List;

import io.reactivex.Completable;
import io.reactivex.Maybe;
import io.reactivex.Observable;
import io.reactivex.Single;

public interface ChatRepository<E extends ChatModel> {

    E newChatModel();

    Observable<List<E>> getAllChats();

    Observable<List<E>> getAllChatsOfAccount(long accountId);

    default Observable<List<E>> getAllChatsOfAccount(AccountModel account) {
        return getAllChatsOfAccount(account.getId());
    }

    Observable<E> getOrCreateChatWith(long accountId, EntityBareJid jid);

    Observable<E> getChatWith(long accountId, EntityBareJid jid);

    default Observable<E> getChatWith(AccountModel accountModel, EntityBareJid jid) {
        return getChatWith(accountModel.getId(), jid);
    }

    Observable<E> getChatWithEntity(long entityId);

    default Observable<E> getChatWithEntity(EntityModel entity) {
        return getChatWithEntity(entity.getId());
    }

    Observable<E> getChatWithContact(long contactId);

    default Observable<E> getChatWithContact(ContactModel contact) {
        return getChatWithContact(contact.getId());
    }

    Maybe<E> maybeGetChatWith(long accountId, EntityBareJid jid);

    default Maybe<E> maybeGetChatWith(AccountModel account, EntityBareJid jid) {
        return maybeGetChatWith(account.getId(), jid);
    }

    Maybe<E> maybeGetChatWithContact(long contactId);

    default Maybe<E> maybeGetChatWithContact(ContactModel contact) {
        return maybeGetChatWithContact(contact.getId());
    }

    Maybe<E> maybeGetChatWithEntity(long entityId);

    default Maybe<E> maybeGetChatWithEntity(EntityModel entity) {
        return maybeGetChatWithEntity(entity.getId());
    }

    Completable updateChat(E chat);

    Single<Long> insertChat(E chat);

    Completable closeChat(E chat);

    Observable<List<Chat>> getDisplayableChats();

}
